# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel

__all__ = ["Query", "Demographics", "File"]


class Demographics(BaseModel):
    date_of_birth: str
    """
    The patient's date of birth, this field is required and must be of the format
    YYYY-MM-DD.
    """

    family_name: str
    """The patient's family name (last name), this field is required."""

    gender: str
    """The patient's gender, must be either Female or Female."""

    given_name: str
    """The patient's given name (first name), this field is required."""

    purpose_of_use: str
    """The purpose of use for which this request is being made"""

    address_city: Optional[str] = None
    """The patient's home city."""

    address_lines: Optional[List[str]] = None
    """The patient's home street address.

    This should not include the patient's city, state, or zip code.
    """

    address_state: Optional[str] = None
    """The patient's home state."""

    email: Optional[str] = None
    """The patient's email, must be a valid email address."""

    encounter_date: Optional[str] = None
    """Encounter Date for which this query is being made"""

    encounter_type: Optional[str] = None
    """Encounter Type for which this query is being made"""

    hints: Optional[List[str]] = None
    """Postal codes hinting at the believed location of records."""

    npi: Optional[str] = None
    """
    The National Provider Identifier of the provider that this request is being made
    by.
    """

    patient_id: Optional[str] = None
    """The unique patient id in an external system"""

    postal_code: Optional[str] = None
    """The patient's home postal (zip) code."""

    specialties: Optional[List[str]] = None
    """The list of specialty searches requested for this patient"""

    ssn: Optional[str] = None
    """
    The patient's social security number, must be a valid US social security number.
    """

    telephone: Optional[str] = None
    """The patient's telephone number."""


class File(BaseModel):
    id: Optional[str] = None
    """Unique ID of the file generated by Particle."""

    authenticator: Optional[str] = None
    """
    Authenticator represents a participant who has attested to the accuracy of the
    document, but who does not have privileges to legally authenticate the document.
    """

    author_name: Optional[str] = None
    """Author Name provided by the source system"""

    author_organization: Optional[str] = None
    """Author Organization provided by the source system."""

    class_code: Optional[str] = None
    """The code specifying the high-level kind of document (e.g.

    Prescription, Discharge Summary, Report, etc.). Reference -
    https://www.hl7.org/fhir/r4/valueset-document-classcodes.html
    """

    confidentiality_code: Optional[str] = None
    """The code specifying the security classification of the document.

    Reference -https://terminology.hl7.org/2.1.0/CodeSystem-v3-Confidentiality.html
    """

    creation_time: Optional[str] = None
    """The timestamp representing when the document was created.

    In the format of RFC3339
    """

    event_code: Optional[str] = None
    """The code specifying the event that resulted in the document being created.

    Reference - https://www.hl7.org/fhir/valueset-event-codes.html
    """

    file_size: Optional[int] = None
    """Size in bytes of the file."""

    file_source: Optional[str] = None
    """FileSource indicates the source network of the file."""

    format_code: Optional[str] = None
    """The code specifying the technical format of the document.

    Reference - https://build.fhir.org/ig/IHE/FormatCode/CodeSystem-formatcode.html
    """

    healthcare_facility_type_code: Optional[str] = None
    """
    The code specifying the type of organizational setting of the clinical encounter
    during which the documented act occurred. Reference -
    https://www.hl7.org/fhir/valueset-c80-facilitycodes.html
    """

    language_code: Optional[str] = None
    """Language of the document.

    Reference - https://build.fhir.org/valueset-languages.html
    """

    practice_setting_code: Optional[str] = None
    """
    The code specifying the clinical specialty where the act that resulted in the
    document was performed (e.g., Family Practice, Laboratory, Radiology).
    Reference - https://www.hl7.org/fhir/valueset-c80-practice-codes.html
    """

    service_start_time: Optional[str] = None
    """The start time of the service being documented. In the format of RFC3339"""

    service_stop_time: Optional[str] = None
    """The stop time of the service being documented. In the format of RFC3339"""

    status_code: Optional[str] = None
    """StatusCode indicates the current status of this document."""

    title: Optional[str] = None
    """Title of the file as provided by the source system."""

    type: Optional[str] = None
    """Mime Type of the file."""

    type_code: Optional[str] = None
    """
    The code specifying the precise type of document from the user perspective
    (e.g., LOINC code). Reference -
    https://www.hl7.org/fhir/r4/valueset-doc-typecodes.html
    """

    url: Optional[str] = None
    """URL path to retrieve the file from."""


class Query(BaseModel):
    id: str

    state: str

    demographics: Optional[Demographics] = None

    expected_completion_time: Optional[str] = None

    files: Optional[List[File]] = None

    start_time: Optional[str] = None
